"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const reflection_helpers_1 = require("./reflection-helpers");
const dependency_container_1 = require("./dependency-container");
function injectable() {
    return function (target) {
        dependency_container_1.typeInfo.set(target, reflection_helpers_1.getParamInfo(target));
    };
}
exports.injectable = injectable;
function singleton() {
    return function (target) {
        injectable()(target);
        dependency_container_1.instance.registerSingleton(target);
    };
}
exports.singleton = singleton;
function autoInjectable() {
    return function (target) {
        const paramInfo = reflection_helpers_1.getParamInfo(target);
        return class extends target {
            constructor(...args) {
                super(...args.concat(paramInfo.slice(args.length).map((type, index) => {
                    try {
                        return dependency_container_1.instance.resolve(type);
                    }
                    catch (e) {
                        const argIndex = index + args.length;
                        const [, params = null] = target.toString().match(/constructor\(([\w, ]+)\)/) || [];
                        const argName = params ? params.split(",")[argIndex] : `#${argIndex}`;
                        throw `Cannot inject the dependency ${argName} of ${target.name} constructor. ${e}`;
                    }
                })));
            }
        };
    };
}
exports.autoInjectable = autoInjectable;
function inject(token) {
    return function (target, _propertyKey, parameterIndex) {
        const injectionTokens = Reflect.getOwnMetadata(reflection_helpers_1.INJECTION_TOKEN_METADATA_KEY, target) || {};
        injectionTokens[parameterIndex] = token;
        Reflect.defineMetadata(reflection_helpers_1.INJECTION_TOKEN_METADATA_KEY, injectionTokens, target);
    };
}
exports.inject = inject;
function registry(registrations = []) {
    return function (target) {
        registrations.forEach((_a) => {
            var { token, options } = _a, provider = tslib_1.__rest(_a, ["token", "options"]);
            return dependency_container_1.instance.register(token, provider, options);
        });
        return target;
    };
}
exports.registry = registry;
