import * as tslib_1 from "tslib";
import { INJECTION_TOKEN_METADATA_KEY, getParamInfo } from "./reflection-helpers";
import { instance as globalContainer, typeInfo } from "./dependency-container";
export function injectable() {
    return function (target) {
        typeInfo.set(target, getParamInfo(target));
    };
}
export function singleton() {
    return function (target) {
        injectable()(target);
        globalContainer.registerSingleton(target);
    };
}
export function autoInjectable() {
    return function (target) {
        const paramInfo = getParamInfo(target);
        return class extends target {
            constructor(...args) {
                super(...args.concat(paramInfo.slice(args.length).map((type, index) => {
                    try {
                        return globalContainer.resolve(type);
                    }
                    catch (e) {
                        const argIndex = index + args.length;
                        const [, params = null] = target.toString().match(/constructor\(([\w, ]+)\)/) || [];
                        const argName = params ? params.split(",")[argIndex] : `#${argIndex}`;
                        throw `Cannot inject the dependency ${argName} of ${target.name} constructor. ${e}`;
                    }
                })));
            }
        };
    };
}
export function inject(token) {
    return function (target, _propertyKey, parameterIndex) {
        const injectionTokens = Reflect.getOwnMetadata(INJECTION_TOKEN_METADATA_KEY, target) || {};
        injectionTokens[parameterIndex] = token;
        Reflect.defineMetadata(INJECTION_TOKEN_METADATA_KEY, injectionTokens, target);
    };
}
export function registry(registrations = []) {
    return function (target) {
        registrations.forEach((_a) => {
            var { token, options } = _a, provider = tslib_1.__rest(_a, ["token", "options"]);
            return globalContainer.register(token, provider, options);
        });
        return target;
    };
}
